/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.data;

import android.content.Context;
import android.database.Cursor;
import android.provider.ContactsContract.CommonDataKinds.Phone;
import android.provider.ContactsContract.Data;
import android.telephony.PhoneNumberUtils;
import android.util.Log;

import java.util.HashMap;

public class Contact {
    // 定义一个静态的HashMap用于缓存联系人信息
    private static HashMap<String, String> sContactCache;
    // 定义一个静态常量TAG用于日志输出
    private static final String TAG = "Contact";
    // 定义一个静态常量CALLER_ID_SELECTION用于查询联系人信息的SQL语句
    private static final String CALLER_ID_SELECTION = "PHONE_NUMBERS_EQUAL(" + Phone.NUMBER
    + ",?) AND " + Data.MIMETYPE + "='" + Phone.CONTENT_ITEM_TYPE + "'"
    + " AND " + Data.RAW_CONTACT_ID + " IN "
            + "(SELECT raw_contact_id "
            + " FROM phone_lookup"
            + " WHERE min_match = '+')";
    // 定义一个静态方法getContact用于获取联系人信息
    public static String getContact(Context context, String phoneNumber) {
        // 如果缓存为空，则初始化缓存
        if(sContactCache == null) {
            sContactCache = new HashMap<String, String>();
        }
        // 如果缓存中已经存在该电话号码的联系人信息，则直接返回
        if(sContactCache.containsKey(phoneNumber)) {
            return sContactCache.get(phoneNumber);
        }
        // 将电话号码转换为最小匹配格式
        String selection = CALLER_ID_SELECTION.replace("+",
                PhoneNumberUtils.toCallerIDMinMatch(phoneNumber));
        // 使用ContentResolver查询联系人信息
        Cursor cursor = context.getContentResolver().query(
                Data.CONTENT_URI,
                new String [] { Phone.DISPLAY_NAME },
                selection,
                new String[] { phoneNumber },
                null);
                
        // 如果查询结果不为空且能够移动到第一行
        if (cursor != null && cursor.moveToFirst()) {
            try {
                // 获取联系人名称
                String name = cursor.getString(0);
                // 将联系人名称存入缓存
                sContactCache.put(phoneNumber, name);
                return name;
            } catch (IndexOutOfBoundsException e) {
                // 如果获取联系人名称时发生异常，输出错误日志
                Log.e(TAG, " Cursor get string error " + e.toString());
                return null;
            } finally {
                // 关闭Cursor
                cursor.close();
            }
        } else {
            // 如果查询结果为空，输出调试日志
            Log.d(TAG, "No contact matched with number:" + phoneNumber);
            return null;
        }
    }
}
