/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.data;

import android.database.Cursor;
import android.util.Log;

import net.micode.notes.tool.GTaskStringUtils;

import org.json.JSONException;
import org.json.JSONObject;


public class MetaData extends Task {
    // 定义一个静态常量TAG，用于日志输出，值为当前类的简单名称
    private final static String TAG = MetaData.class.getSimpleName();

    // 定义一个私有成员变量mRelatedGid，用于存储相关的GID
    private String mRelatedGid = null;

    // 设置元数据的方法，接收一个GID和一个JSONObject类型的metaInfo
    public void setMeta(String gid, JSONObject metaInfo) {
        try {
            // 将GID放入metaInfo的META_HEAD_GTASK_ID键中
            metaInfo.put(GTaskStringUtils.META_HEAD_GTASK_ID, gid);
        } catch (JSONException e) {
            // 如果放入失败，输出错误日志
            Log.e(TAG, "failed to put related gid");
        }
        // 将metaInfo转换为字符串并设置为任务的笔记
        setNotes(metaInfo.toString());
        // 设置任务的名称为META_NOTE_NAME
        setName(GTaskStringUtils.META_NOTE_NAME);
    }

    // 获取相关GID的方法
    public String getRelatedGid() {
        return mRelatedGid;
    }

    // 重写isWorthSaving方法，判断任务是否值得保存
    @Override
    public boolean isWorthSaving() {
        // 如果笔记不为空，则返回true
        return getNotes() != null;
    }

    // 重写setContentByRemoteJSON方法，根据远程JSON设置内容
    @Override
    public void setContentByRemoteJSON(JSONObject js) {
        // 调用父类的setContentByRemoteJSON方法
        super.setContentByRemoteJSON(js);
        // 如果笔记不为空
        if (getNotes() != null) {
            try {
                // 将笔记转换为JSONObject
                JSONObject metaInfo = new JSONObject(getNotes().trim());
                // 从metaInfo中获取META_HEAD_GTASK_ID的值并赋给mRelatedGid
                mRelatedGid = metaInfo.getString(GTaskStringUtils.META_HEAD_GTASK_ID);
            } catch (JSONException e) {
                // 如果转换失败，输出警告日志
                Log.w(TAG, "failed to get related gid");
                // 将mRelatedGid设置为null
                mRelatedGid = null;
            }
        }
    }

    // 重写setContentByLocalJSON方法，该方法不应被调用
    @Override
    public void setContentByLocalJSON(JSONObject js) {
        // this function should not be called
        throw new IllegalAccessError("MetaData:setContentByLocalJSON should not be called");
    }

    @Override
    public JSONObject getLocalJSONFromContent() {
        throw new IllegalAccessError("MetaData:getLocalJSONFromContent should not be called");
    }

    @Override
    public int getSyncAction(Cursor c) {
        throw new IllegalAccessError("MetaData:getSyncAction should not be called");
    }

}
