/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.data;

import android.database.Cursor;

import org.json.JSONObject;

// 定义一个抽象类 Node，用于表示节点的基本属性和行为
public abstract class Node {
    // 定义同步操作的常量，表示无操作
    public static final int SYNC_ACTION_NONE = 0;

    // 定义同步操作的常量，表示添加远程节点
    public static final int SYNC_ACTION_ADD_REMOTE = 1;

    // 定义同步操作的常量，表示添加本地节点
    public static final int SYNC_ACTION_ADD_LOCAL = 2;

    // 定义同步操作的常量，表示删除远程节点
    public static final int SYNC_ACTION_DEL_REMOTE = 3;

    // 定义同步操作的常量，表示删除本地节点
    public static final int SYNC_ACTION_DEL_LOCAL = 4;

    // 定义同步操作的常量，表示更新远程节点
    public static final int SYNC_ACTION_UPDATE_REMOTE = 5;

    // 定义同步操作的常量，表示更新本地节点
    public static final int SYNC_ACTION_UPDATE_LOCAL = 6;

    // 定义同步操作的常量，表示更新冲突
    public static final int SYNC_ACTION_UPDATE_CONFLICT = 7;

    // 定义同步操作的常量，表示错误
    public static final int SYNC_ACTION_ERROR = 8;

    // 私有成员变量，表示节点的全局唯一标识符
    private String mGid;

    // 私有成员变量，表示节点的名称
    private String mName;

    // 私有成员变量，表示节点的最后修改时间
    private long mLastModified;

    // 私有成员变量，表示节点是否被删除
    private boolean mDeleted;

    // 构造函数，初始化节点的基本属性
    public Node() {
        mGid = null; // 初始化全局唯一标识符为空
        mName = ""; // 初始化名称为空字符串
        mLastModified = 0; // 初始化最后修改时间为0
        mDeleted = false; // 初始化删除状态为未删除
    }

    // 抽象方法，用于获取创建操作的JSON对象
    public abstract JSONObject getCreateAction(int actionId);

    // 抽象方法，用于获取更新操作的JSON对象
    public abstract JSONObject getUpdateAction(int actionId);

    // 抽象方法，根据远程JSON对象设置节点内容
    public abstract void setContentByRemoteJSON(JSONObject js);

    // 抽象方法，根据本地JSON对象设置节点内容
    public abstract void setContentByLocalJSON(JSONObject js);

    // 抽象方法，从节点内容获取本地JSON对象
    public abstract JSONObject getLocalJSONFromContent();

    // 抽象方法，根据游标获取同步操作类型
    public abstract int getSyncAction(Cursor c);

    // 设置节点的全局唯一标识符
    public void setGid(String gid) {
        this.mGid = gid;
    }

    // 设置节点的名称
    public void setName(String name) {
        this.mName = name;
    }

    // 设置节点的最后修改时间
    public void setLastModified(long lastModified) {
        this.mLastModified = lastModified;
    }

    // 设置节点的删除状态
    public void setDeleted(boolean deleted) {
        this.mDeleted = deleted;
    }

    // 获取节点的全局唯一标识符
    public String getGid() {
        return this.mGid;
    }

    // 获取节点的名称
    public String getName() {
        return this.mName;
    }

    // 获取节点的最后修改时间
    public long getLastModified() {
        return this.mLastModified;
    }

    // 获取节点的删除状态
    public boolean getDeleted() {
        return this.mDeleted;
    }

}
