/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.tool;

import android.content.Context;
import android.preference.PreferenceManager;

import net.micode.notes.R;
import net.micode.notes.ui.NotesPreferenceActivity;

public class ResourceParser {

    public static final int YELLOW           = 0;
    public static final int BLUE             = 1;
    public static final int WHITE            = 2;
    public static final int GREEN            = 3;
    public static final int RED              = 4;

    public static final int BG_DEFAULT_COLOR = YELLOW;

    public static final int TEXT_SMALL       = 0;
    public static final int TEXT_MEDIUM      = 1;
    public static final int TEXT_LARGE       = 2;
    public static final int TEXT_SUPER       = 3;

    public static final int BG_DEFAULT_FONT_SIZE = TEXT_MEDIUM;

    // 定义一个名为NoteBgResources的公共静态类，用于管理音符和笔记标题的背景资源
    public static class NoteBgResources {
        // 定义一个私有的静态整型数组，用于存储编辑背景资源的ID
        private final static int [] BG_EDIT_RESOURCES = new int [] {
            R.drawable.edit_yellow,  // 黄色编辑背景资源ID
            R.drawable.edit_blue,    // 蓝色编辑背景资源ID
            R.drawable.edit_white,   // 白色编辑背景资源ID
            R.drawable.edit_green,   // 绿色编辑背景资源ID
            R.drawable.edit_red      // 红色编辑背景资源ID
        };

        // 定义一个私有的静态整型数组，用于存储编辑标题背景资源的ID
        private final static int [] BG_EDIT_TITLE_RESOURCES = new int [] {
            R.drawable.edit_title_yellow,  // 黄色编辑标题背景资源ID
            R.drawable.edit_title_blue,    // 蓝色编辑标题背景资源ID
            R.drawable.edit_title_white,   // 白色编辑标题背景资源ID
            R.drawable.edit_title_green,   // 绿色编辑标题背景资源ID
            R.drawable.edit_title_red      // 红色编辑标题背景资源ID
        };

        // 定义一个公共的静态方法，用于获取音符背景资源的ID
        public static int getNoteBgResource(int id) {
            // 通过传入的id参数，从BG_EDIT_RESOURCES数组中获取对应的资源ID并返回
            return BG_EDIT_RESOURCES[id];
        }

        // 定义一个公共的静态方法，用于获取笔记标题背景资源的ID
        public static int getNoteTitleBgResource(int id) {
            // 通过传入的id参数，从BG_EDIT_TITLE_RESOURCES数组中获取对应的背景资源ID
            return BG_EDIT_TITLE_RESOURCES[id];
        }
    }

    // 定义一个静态方法，用于获取默认的背景ID
    public static int getDefaultBgId(Context context) {
        // 从PreferenceManager中获取默认的SharedPreferences实例
        // 然后根据键值NotesPreferenceActivity.PREFERENCE_SET_BG_COLOR_KEY获取布尔值
        // 如果该键值对应的值为true，则执行if语句块中的代码
        if (PreferenceManager.getDefaultSharedPreferences(context).getBoolean(
                NotesPreferenceActivity.PREFERENCE_SET_BG_COLOR_KEY, false)) {
            // 生成一个0到NoteBgResources.BG_EDIT_RESOURCES.length之间的随机整数
            // 作为背景资源的索引，并返回该索引值
            return (int) (Math.random() * NoteBgResources.BG_EDIT_RESOURCES.length);
        } else {
            // 如果该键值对应的值为false，则返回默认的背景颜色常量BG_DEFAULT_COLOR
            return BG_DEFAULT_COLOR;
        }
    }

    // 定义一个静态内部类 NoteItemBgResources，用于管理笔记项的背景资源
    public static class NoteItemBgResources {
        // 定义一个静态常量数组，存储首项背景资源ID
        private final static int [] BG_FIRST_RESOURCES = new int [] {
            R.drawable.list_yellow_up,    // 黄色背景，向上箭头
            R.drawable.list_blue_up,      // 蓝色背景，向上箭头
            R.drawable.list_white_up,     // 白色背景，向上箭头
            R.drawable.list_green_up,     // 绿色背景，向上箭头
            R.drawable.list_red_up        // 红色背景，向上箭头
        };

        // 定义一个静态常量数组，存储普通项背景资源ID
        private final static int [] BG_NORMAL_RESOURCES = new int [] {
            R.drawable.list_yellow_middle,    // 黄色背景，中间部分
            R.drawable.list_blue_middle,      // 蓝色背景，中间部分
            R.drawable.list_white_middle,     // 白色背景，中间部分
            R.drawable.list_green_middle,     // 绿色背景，中间部分
            R.drawable.list_red_middle        // 红色背景，中间部分
        };

        // 定义一个静态常量数组，存储末项背景资源ID
        private final static int [] BG_LAST_RESOURCES = new int [] {
            R.drawable.list_yellow_down,    // 黄色背景，向下箭头
            R.drawable.list_blue_down,      // 蓝色背景，向下箭头
            R.drawable.list_white_down,     // 白色背景，向下箭头
            R.drawable.list_green_down,
            R.drawable.list_red_down,
        };

        private final static int [] BG_SINGLE_RESOURCES = new int [] {
            R.drawable.list_yellow_single,
            R.drawable.list_blue_single,
            R.drawable.list_white_single,
            R.drawable.list_green_single,
            R.drawable.list_red_single
        };

        // 定义一个公共的静态方法，用于获取音符背景的第一个资源ID
        public static int getNoteBgFirstRes(int id) {
            // 通过传入的id参数，从BG_FIRST_RESOURCES数组中获取对应的资源ID并返回
            return BG_FIRST_RESOURCES[id];
        }

        // 定义一个公共的静态方法，用于获取音符背景的最后资源ID
        public static int getNoteBgLastRes(int id) {
            // 通过传入的id参数，从BG_LAST_RESOURCES数组中获取对应的资源ID并返回
            return BG_LAST_RESOURCES[id];
        }

        // 定义一个公共的静态方法，用于获取单个音符背景资源的ID
        public static int getNoteBgSingleRes(int id) {
            // 通过传入的id参数，从BG_SINGLE_RESOURCES数组中获取对应的资源ID并返回
            return BG_SINGLE_RESOURCES[id];
        }

        // 定义一个公共的静态方法，用于获取普通背景资源的ID
        public static int getNoteBgNormalRes(int id) {
            // 通过传入的id参数，从BG_NORMAL_RESOURCES数组中获取对应的背景资源ID
            return BG_NORMAL_RESOURCES[id];
        }

        // 定义一个公共的静态方法，用于获取文件夹背景资源的ID
        public static int getFolderBgRes() {
            // 返回R.drawable资源类中名为list_folder的资源ID
            return R.drawable.list_folder;
        }
    }

    // 定义一个名为WidgetBgResources的静态内部类
    public static class WidgetBgResources {
        // 定义一个静态的final整型数组，用于存储2x尺寸的背景资源ID
        private final static int [] BG_2X_RESOURCES = new int [] {
            R.drawable.widget_2x_yellow,  // 2x尺寸的黄色背景资源
            R.drawable.widget_2x_blue,    // 2x尺寸的蓝色背景资源
            R.drawable.widget_2x_white,   // 2x尺寸的白色背景资源
            R.drawable.widget_2x_green,   // 2x尺寸的绿色背景资源
            R.drawable.widget_2x_red,     // 2x尺寸的红色背景资源
        };

        // 定义一个静态方法，用于根据ID获取2x尺寸的背景资源
        public static int getWidget2xBgResource(int id) {
            return BG_2X_RESOURCES[id];  // 返回对应ID的背景资源
        }

        // 定义一个静态的final整型数组，用于存储4x尺寸的背景资源ID
        private final static int [] BG_4X_RESOURCES = new int [] {
            R.drawable.widget_4x_yellow,  // 4x尺寸的黄色背景资源
            R.drawable.widget_4x_blue,    // 4x尺寸的蓝色背景资源
            R.drawable.widget_4x_white,   // 4x尺寸的白色背景资源
            R.drawable.widget_4x_green,   // 4x尺寸的绿色背景资源
            R.drawable.widget_4x_red      // 4x尺寸的红色背景资源
        };

        // 定义一个静态方法，用于根据ID获取4x尺寸的背景资源
        public static int getWidget4xBgResource(int id) {
            return BG_4X_RESOURCES[id];  // 返回对应ID的背景资源
        }
    }

    public static class TextAppearanceResources {
        // 定义一个静态数组，用于存储文本外观的资源ID
        private final static int [] TEXTAPPEARANCE_RESOURCES = new int [] {
            R.style.TextAppearanceNormal,    // 正常文本外观样式
            R.style.TextAppearanceMedium,   // 中等文本外观样式
            R.style.TextAppearanceLarge,    // 大文本外观样式
            R.style.TextAppearanceSuper     // 超大文本外观样式
        };

        /**
         * 根据给定的ID获取对应的文本外观资源
         * @param id 文本外观资源的索引ID
         * @return 对应的文本外观资源ID，如果ID超出范围则返回默认字体大小资源
         */
        public static int getTexAppearanceResource(int id) {
            /**
             * HACKME: Fix bug of store the resource id in shared preference.
             * The id may larger than the length of resources, in this case,
             * return the {@link ResourceParser#BG_DEFAULT_FONT_SIZE}
             */
            if (id >= TEXTAPPEARANCE_RESOURCES.length) {
                return BG_DEFAULT_FONT_SIZE;
            }
            return TEXTAPPEARANCE_RESOURCES[id];
        }

        public static int getResourcesSize() {
            return TEXTAPPEARANCE_RESOURCES.length;
        }
    }
}
